package Bio::IO::CBust::Record;

sub new {
  my $class = shift;
  my $self = { @_ };
  bless $self, $class;
  return $self;
}
sub add_cluster {
  my $self = shift;
  push @{$self->{clusters}}, @_;
}
sub clusters {
  my $self = shift;
  if(@_) {
    $self->{clusters} = shift;
  }
  return $self->{clusters};


}

sub gene {
  my $self = shift;
  if(@_) {
    $self->{gene} = shift;
  }
  return $self->{gene};

}

sub len {
  my $self = shift;
  if(@_) {
    $self->{len} = shift;
  }
  return $self->{len};

}

sub table {
  my $self = shift;
  my @lines = join "\t", ('Gene',
     'Cluster',
     'Location',
     'Cluster score',
     'matrix',
     'location start',
     'location stop',
     'strand',
     'matrix',
     'score',
     'sequences');
  my @r;
  																																																																																																																																																																																																																																																			
  foreach my $cluster (@{$self->{clusters}}) {
    my $cn = $cluster->name;
    $cn =~ s/CLUSTER\s*//;
    @r = ($self->gene, 
          $cn, 
	  (join " to ", @{$cluster->location}), 
          $cluster->score);
    foreach my $matrix (@{$cluster->matrix}) {
      my @t;
      foreach (qw(name start stop strand score sequence)) {
        push @t, $matrix->$_();
      }
      push @lines, join "\t", (@r, @t);
    }
  }
  return ((join "\n", @lines) . "\n");
}

sub load_sqlite {
  my $self = shift;
  my $file = shift;
  use DBI;
  my $dbh = DBI->connect("dbi:SQLite:dbname=$file", "", "") or do { print STDERR DBI->errstr; return; };
  if((! -f "$file") || (-z "$file")) {
    # Create the database
    $dbh->do("CREATE TABLE SITE (SITE VARCHAR(256) PRIMARY KEY)");
    $dbh->do("CREATE TABLE GENE (GENE VARCHAR(64) PRIMARY KEY, BP INTEGER)");
    $dbh->do("CREATE TABLE CLUSTER (CLUSTER_ID INTEGER PRIMARY KEY, GENE VARCHAR(64), NAME VARCHAR(64), START INTEGER, STOP INTEGER, SCORE FLOAT)");
    $dbh->do("CREATE TABLE MOTIF (MOTIF_ID INTEGER PRIMARY KEY, CLUSTER_ID INTEGER, MATRIX VARCHAR(256), START INTEGER, STOP INTEGER, STRAND VARCHAR(1), SCORE FLOAT, SEQUENCE TEXT, SITE VARCHAR(256))"); 
  }
  $dbh->{AutoCommit} = 1;
  my $csth = $dbh->prepare('INSERT INTO CLUSTER (CLUSTER_ID, GENE, NAME, START, STOP, SCORE) VALUES (?, ?, ?, ?, ?, ?)');
  my $msth = $dbh->prepare('INSERT INTO MOTIF (MOTIF_ID, CLUSTER_ID, MATRIX, START, STOP, STRAND, SCORE, SEQUENCE, SITE) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)');
  my $lsth = $dbh->prepare('select last_insert_rowid()');
  foreach my $cluster (@{$self->{clusters}}) {
    $dbh->do('REPLACE INTO GENE (GENE) VALUES (' . "'" . $self->gene . "'" . ')');
    my $cn = $cluster->name;
    $cn =~ s/CLUSTER\s*//;
    $csth->execute(undef, $self->gene, $cn, @{$cluster->location}, $cluster->score);
    $lsth->execute;
    my($cid) = $lsth->fetchrow();
    foreach my $matrix (@{$cluster->matrix}) {
      my @t;
      $dbh->do('REPLACE INTO SITE (SITE) VALUES (' . "'" . $matrix->site . "'" . ')');
      foreach (qw(name start stop strand score sequence site)) {
        push @t, $matrix->$_();
      }
      $msth->execute(undef, $cid, @t);
    }
  }
  
}

=pod

=item search_motif_group

Search the record for motif group.

PARAMS: $motif_group - list of motif in arrayref.
RETURNS: hash ref shown below

REGION => {
  $pattern0 => $count0,
  $pattern1 => $count1,
  ...
},
...


E.g.

'1082 to 1136' => {
  OCT1 => 5,
  CEBP => 3
}

=cut

sub search_motif_group {
  my $self = shift;
  my $motif_group = shift;
  my %r;
  foreach my $cluster (@{$self->{clusters}}) {
    my %mh = map { $_ => 0 } @$motif_group;
    foreach my $matrix (@{$cluster->matrix}) {
      my $site = $matrix->site;
      $mh{$site} ++ if(defined $mh{$site});
    }
    unless(grep { $_ == 0 } values %mh) {
      my $location = join " to ", @{$cluster->location};
      $r{$location} = \%mh;
    }
  }
  return \%r;
}

=pod

=item patterns

Generate all pattern with the cutoff.

PARAMS: cutoff - number hit minimum to be consider a hit (default is 2)
RETURNS: Array of Patterns.

=cut

sub patterns {
  my $self = shift;
  my %param = @_;
  my $cutoff = $param{cutoff};
  my $cluster_score_cutoff = $param{cluster_score_cutoff};
  my $matrix_score_cutoff = $param{matrix_score_cutoff};
  $cluster_score_cutoff ||= 0;
  $cutoff ||= 2;
  unless($cutoff > 0 && $cutoff =~ /^\d+$/) {
    print STDERR "ERROR: Parameter must be an integer greater than 0!\n";
    return;
  }
  my @patterns;
  foreach my $cluster (@{$self->{clusters}}) {
    next if($cluster->score < $cluster_score_cutoff);
    my %mh;
    foreach my $matrix (@{$cluster->matrix}) {
      next if($matrix->score < $matrix_score_cutoff);
      $mh{$matrix->site} ++;
    }
    my @pp = grep { $mh{$_} >= $cutoff } sort keys %mh;
    if(@pp > 1) {
      my $n = $self->_generate_combination_patterns(@pp);
      while(my @dd = $n->()) {
        push @patterns, (join "_", @dd) if(@dd > 1);
      }
    }
  }
  return sort keys %{{ map { $_ => 1 } @patterns }};
}

=pod

=item sites

Get all sites with the cutoff.

PARAMS: cutoff - number hit minimum to be consider a hit (default is 2)
RETURNS: Hash of cluster id of Array of Sites.

=cut

sub sites {
  my $self = shift;
  my %param = @_;
  my $cutoff = $param{cutoff};
  my $cluster_score_cutoff = $param{cluster_score_cutoff};
  my $matrix_score_cutoff = $param{matrix_score_cutoff};
  $cluster_score_cutoff ||= 0;
  $cutoff ||= 2;
  unless($cutoff > 0 && $cutoff =~ /^\d+$/) {
    print STDERR "ERROR: Parameter must be an integer greater than 0!\n";
    return;
  }
  my @patterns;
  my %all;
  my $id = 0;
  foreach my $cluster (@{$self->{clusters}}) {
    next if($cluster->score < $cluster_score_cutoff);
    my %mh;
    foreach my $matrix (@{$cluster->matrix}) {
      next if($matrix->score < $matrix_score_cutoff);
      $mh{$matrix->site} ++;
    }
    my @pp = grep { $mh{$_} >= $cutoff } sort keys %mh;
    if(@pp) {
      $id ++;
      push @{$all{$id}}, @pp;
    }
  }
  return \%all;
}

sub _generate_combination_patterns {
  my $self = shift;
  my @list = @_;
  return sub { () } unless(@list);  
  my $count = 0;
  if(scalar(@list) > 15) {
    die "The list [" . scalar(@list) . " elements] is bigger than 15 elements! Gene [" . $self->gene . "]\n";
  }
  my $max = (2 ** scalar(@list)) - 1;
  return sub {
    return () if($count >= $max);
    $count ++;
    my $c = $count;
    my $i = 0;
    my @r;
    if($c & 1) {
      push @r, $list[$i];
    }
    while($c = $c >> 1) {    
      $i ++;
      if($c & 1) {
	push @r, $list[$i];
      }
    }
    return @r;
 };
}
sub generate_combination_patterns {
  my $proto = shift;
  my @list = @_;
  return sub { () } unless(@list);  
  my $count = 0;
  if(scalar(@list) > 15) {
    die "The list [" . scalar(@list) . " elements] is bigger than 15 elements!";
  }
  my $max = (2 ** scalar(@list)) - 1;
  return sub {
    return () if($count >= $max);
    $count ++;
    my $c = $count;
    my $i = 0;
    my @r;
    if($c & 1) {
      push @r, $list[$i];
    }
    while($c = $c >> 1) {    
      $i ++;
      if($c & 1) {
	push @r, $list[$i];
      }
    }
    return @r;
 };
}
1;
