#!/usr/bin/perl

use strict;
use warnings;
use File::Basename;
use IO::File;
use Getopt::Long;
use Bio::IO::CBust;

our ($file, $motif, $mfile, $output_dir, $sfile, $cluster_score_cutoff, $matrix_score_cutoff); 

my $result = GetOptions ("motif=s" => \$motif,
                         "motif-file=s" => \$mfile,
                         "file=s"   => \$file,
                         "sqlite=s"   => \$sfile,
                         "cluster-score-cutoff=f"   => \$cluster_score_cutoff,
                         "matrix-score-cutoff=f"   => \$matrix_score_cutoff,
                         "output-dir=s"  => \$output_dir);

unless((-f "$mfile" || $motif) && -f "$file" && -d "$output_dir") {
  unless(-f "$mfile" || $motif) {
    print "--motif-file [$mfile] must be a valid file\n"; 
    print "or\n--motif [$motif] must be a valid motif\n"; 
  }
  unless(-f "$file") {
    print "--file [$file] must be a valid file\n"; 
  }
  unless(-d "$output_dir") {
    print "--output-dir [$output_dir] must be a valid directory\n"; 
  }
  die <<EOT;

Usage:

$0 
          --file <cluster-buster output file>
          [--motif <motif to use to search the outputfile> |
          --motif-file <motif file to use to search the outputfile>]
          --output-dir <output directory>
          [--cluster-score-cutoff <cluster score cutoff>]
          [--matrix-score-cutoff <matrix score cutoff>]
	  [--sqlite <SQLite database filename>]
          

 --file        Cluster-buster output file
 --motif       Motif to use to search the outputfile
 --motif-file  Motif file to use to search the outputfile
 --output-dir  Output directory
 --cluster-score-cutoff cluster score cutoff (default 0)
 --matrix-score-cutoff matrix score cutoff (default 0)
 --sqlite      Generate the sqlite database output with the filename specified

EOT

}

my $bn = basename($file);
my $first_out = "$output_dir/$bn.ff";
my $second_out = "$output_dir/$bn.sf";
my $third_out = "$output_dir/$bn.tf";

open(FO, ">$first_out") or die "ERROR[$first_out]: $!";
open(SO, ">$second_out") or die "ERROR[$second_out]: $!";
open(TO, ">$third_out") or die "ERROR[$third_out]: $!";

my @motif_group;
if(-f "$mfile") {
 push @motif_group, read_motif($mfile);
}
if($motif) {
  push @motif_group, split /\,/, $motif;
}
my $gc = new Bio::IO::CBust(file => $file);
my %third;
my $ng = 0;
while(my $r = $gc->next_record) {
  print FO $r->table;
  $ng ++;
  my $gene = $r->gene;
  foreach my $g (@motif_group) {
    my $mr = $r->search_motif_group($g);
    
    foreach my $location (keys %$mr) {
      print SO join "\t", ($gene, $location, (join '_', @$g), (join '_', map { $mr->{$location}->{$_} } @$g));
      print SO "\n";
    }
  }
  foreach my $p ($r->patterns(matrix_score_cutoff => $matrix_score_cutoff, cluster_score_cutoff => $cluster_score_cutoff)) {
    $third{$p}->{$gene} = 1;
  }
  if($sfile) {
    $r->load_sqlite($sfile);
  }
}
foreach my $p (sort keys %third) {
  my @gl = sort keys %{$third{$p}};
  print TO join "\t", ("$p", scalar(@gl), (join ',', sort keys %{$third{$p}}), sprintf("%0.2f", 100 * scalar(@gl)/$ng));
  print TO "\n";
}
close FO;
close SO;
close TO;

sub read_motif {
  my $mf = shift;
  open(FILE, "$mf") or die "ERROR[$mf]: $!";
  my $mgc = 0;
  my @g;
  while(<FILE>) {
    s/^\s*|\s*$//g;
    next unless($_);
    push @g, [split /\_/, $_];
  }
  return @g;
}

=pod

=item NAME

cbusta.pl - Post processing for the Cluster-Buster output.

=item SYNOSIS

cbusta.pl --file <cluster-buster output file>
          --motif <motif to use to search the outputfile>
          --output-dir <output directory>
	  [--sqlite <SQLite database filename>]
          

 --file        Cluster-buster output file
 --motif       Motif to use to search the outputfile
 --output-dir  Output directory
 --sqlite      Generate the sqlite database output with the filename specified


=item DESCRIPTION

Post processing for the Cluster-Bluster output.


